{%- extends "basic/layout.html" %}
{%- block extrahead %}
    {{ super() }}
        {% if theme_touch_icon %}
    <link rel="apple-touch-icon" href="{{ pathto('_static/' ~ theme_touch_icon, 1) }}" />
        {% endif %}
    <link media="only screen and (max-device-width: 480px)" href="{{pathto('_static/small_groundwork.css', 1) }}" type= "text/css" rel="stylesheet" />
    <meta name="viewport" content="width=device-width, initial-scale=0.9, maximum-scale=0.9">
    <!-- Place this tag in your head or just before your close body tag. -->
{% endblock %}

{%- block sidebarsearch %}
    {{ super() }}
    {%- include "contribute.html" %}
{% endblock %}

{% block relbar2 %}
 {% if theme_github_fork %}
        <div class="github_fork">
            <a href="http://github.com/{{ theme_github_fork }}">
                <img style="position: fixed; top: 0; right: 0; border: 0;"
                     src="https://s3.amazonaws.com/github/ribbons/forkme_right_{{ theme_github_ribbon_color }}.png"
                     alt="Fork me on GitHub">
            </a>
        </div>
    {% endif %}

{% endblock %}

{%- block footer %}
    <div class="footer">
      &copy; Copyright {{ copyright }}.
    </div>
{%- endblock %}
